/*

  ShowChannels() v0.0 Plugin for Avisynth v2.5

	Copyright (c) 2009 by Stainless Stephen (Jones)

  Some Parts (c)
	thejam79
	Donald Graft.
	Violao
*/

/*
	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

*/

#define ShowChannelsVer 0.0

#include "internal.h"
#include "info.h"

#define GETFRAME(g, fp) \
{ \
	int GETFRAMEf; \
	GETFRAMEf = (g); \
	if (GETFRAMEf < 0) GETFRAMEf = 0; \
	if (GETFRAMEf > num_frames_hi - 1) GETFRAMEf = num_frames_hi - 1; \
	(fp) = child->GetFrame(GETFRAMEf, env); \
}


class ShowChannels : public GenericVideoFilter {

bool	ver;
int		num_frames_hi;
float	FChan[3];
float	AChan[3];
int		Frames_Visited;

	void __stdcall DrawShow(PVideoFrame &dst, int n, IScriptEnvironment* env);
    void __stdcall DrawString(PVideoFrame &dst, int x, int y, bool color, const char *s);
public:
	ShowChannels(PClip _child,bool _ver,  IScriptEnvironment* env);
	~ShowChannels(){};
	PVideoFrame __stdcall GetFrame(int n, IScriptEnvironment* env);
};

ShowChannels::ShowChannels(PClip _child,bool _ver, IScriptEnvironment* env) : \
		GenericVideoFilter(_child), ver(_ver)
{

	if (!vi.IsYV12() && !vi.IsYUY2())
		env->ThrowError("ShowChannels: Requires YV12 or YUY2");

	num_frames_hi = vi.num_frames;
	Frames_Visited = 0;
	AChan[0]=AChan[1]=AChan[2]=0.0;
}


// DrawShow(), Called only by GetFrame()
void ShowChannels::DrawShow(PVideoFrame &dst, int n, IScriptEnvironment* env) {
	char buf[80];
	env->MakeWritable(&dst);

    int voff = 0;

	if(ver) {
		sprintf(buf,"ShowChannels() v%.1f  %c 2009",ShowChannelsVer,137);
		DrawString(dst, 0, voff+0,false, buf);
		sprintf(buf,"By Stainless Stephen (Jones)");
		DrawString(dst, 0, voff+1,false, buf);
		voff+=3;
	}

	sprintf(buf,"Y = %.2f (%.2f)",FChan[0],AChan[0] / Frames_Visited);
	DrawString(dst, 0, voff+0,false, buf);
	sprintf(buf,"U = %.2f (%.2f)",FChan[1],AChan[1] / Frames_Visited);
	DrawString(dst, 0, voff+1,false, buf);
	sprintf(buf,"V = %.2f (%.2f)",FChan[2],AChan[2] / Frames_Visited);
	DrawString(dst, 0, voff+2,false, buf);

	voff+=3;
}

void __stdcall ShowChannels::DrawString(PVideoFrame &dst, int x, int y, bool color, const char *s) 
{
extern void DrawDigitYUY2(PVideoFrame &dst, int x, int y,bool color, int num);
extern void DrawDigitYV12(PVideoFrame &dst, int x, int y,bool color, int num);

	if (vi.IsYV12()) {
		for (int xx = 0; *s; ++s, ++xx) {
			DrawDigitYV12(dst, x + xx, y, color, *s - ' ');
		}

	} else {
		for (int xx = 0; *s; ++s, ++xx) {
			DrawDigitYUY2(dst, x + xx, y, color, *s - ' ');
		}
	}
}


PVideoFrame __stdcall ShowChannels::GetFrame(int n, IScriptEnvironment* env) {
    PVideoFrame src;

	if(n==0) {	// Reset Stats
		Frames_Visited		= 0;
		AChan[0]=AChan[1]=AChan[2]=0.0;
	}
	++Frames_Visited;
	FChan[0]=FChan[1]=FChan[2]=0.0;

	GETFRAME(n,src);
	
	if(vi.IsYUY2()) {
	    const int   pitch   = src->GetPitch();
	    const int   rowsize = src->GetRowSize();
	    const int   height  = src->GetHeight();
	    const BYTE *srcp	= src->GetReadPtr();
  	
		for (int h=0; h<height; h+=1) {
			for (int w=0; w<rowsize; w+=4) {
				FChan[0] +=	srcp[h*pitch + w + 0] + srcp[h*pitch + w + 2]; // y, 2 pixels
				FChan[1] +=	srcp[h*pitch + w + 1];	// u
				FChan[2] +=	srcp[h*pitch + w + 3];	// v
			}

		}
		FChan[0] /= (height * rowsize / 2);
		FChan[1] /= (height * rowsize / 4);
		FChan[2] /= (height * rowsize / 4);
	} else {

		for(int pn=0;pn<3;++pn) {
			int yv12Plane=(pn==0)?PLANAR_Y:(pn==1)?PLANAR_U:PLANAR_V;
			const int   pitch   = src->GetPitch(yv12Plane);
			const int   rowsize = src->GetRowSize(yv12Plane);
			const int   height  = src->GetHeight(yv12Plane);
			const BYTE *srcp	= src->GetReadPtr(yv12Plane);
	
			for (int h=0; h<height; h+=1) {
				for (int w=0; w<rowsize; w+=1) {
					FChan[pn] +=	srcp[h*pitch + w];
				}
			}
			FChan[pn] /= (height * rowsize);
		}
	}
	AChan[0] += FChan[0];	// y
	AChan[1] += FChan[1];	// u
	AChan[2] += FChan[2];	// v
	DrawShow(src,n,env);
    return src;
}

AVSValue __cdecl Create_ShowChannels(AVSValue args, void* user_data, IScriptEnvironment* env) {
    return new ShowChannels(args[0].AsClip(),args[1].AsBool(false), env);  
}

extern "C" __declspec(dllexport) const char* __stdcall AvisynthPluginInit2(IScriptEnvironment* env) {
    env->AddFunction("ShowChannels", "c[ver]b", Create_ShowChannels, 0);
    return "`ShowChannels' ShowChannels plugin";
}
